<?php

namespace Drupal\template_diff\Commands;

use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drush\Commands\DrushCommands;
use SebastianBergmann\Diff\Differ;
use SebastianBergmann\Diff\Output\UnifiedDiffOutputBuilder;
use Symfony\Component\Finder\Finder;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml
 */
class TemplateDiffCommands extends DrushCommands {

  /**
   * Theme handler service.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * Theme manager service.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

  /**
   * Builds the template diff service.
   */
  public function __construct(ThemeHandlerInterface $themeHandler, ThemeManagerInterface $themeManager) {
    $this->themeHandler = $themeHandler;
    $this->themeManager = $themeManager;
  }

  /**
   * Displays the diff between two templates files in specified themes.
   *
   * @param string $template
   *   The template name.
   * @param string $theme_1
   *   The first theme name to compare.
   * @param string $theme_2
   *   The second theme name to compare.
   *
   * @usage template_diff:show node foo_theme bar_theme
   *   Compare the node template in two themes
   *
   * @command template_diff:show
   */
  public function commandName(string $template, string $theme_1 = NULL, string $theme_2 = NULL) {
    // Build the template name.
    $template = sprintf('%s.html.twig', $template);
    // Determine the active theme.
    $active_theme = $this->themeManager->getActiveTheme()->getName();

    // Compare the two themes that were passed in as arguments.
    if (!empty($theme_1) && !empty($theme_2)) {
      // Log the themes names being compared and how we determined them.
      $this->logger->notice(dt('Comparing :theme_1 and :theme_2.', [
        ':theme_1' => $theme_1,
        ':theme_2' => $theme_2,
      ]));
    }
    // If only one theme was supplied, use the active theme as the second one.
    elseif (!empty($theme_1) && empty($theme_2)) {
      $theme_2 = $active_theme;
      // Log the themes names being compared and how we determined them.
      $this->logger->notice(dt('Comparing :theme_1 and the active theme (:theme_2).', [
        ':theme_1' => $theme_1,
        ':theme_2' => $theme_2,
      ]));
    }
    // Compare the active theme and its parent theme.
    else {
      // Set the active theme as the first theme.
      $theme_1 = $active_theme;
      // Find the parent theme. If no base theme is set, the theme will
      // inherit from core's stable theme by default.
      $theme_2 = $this->themeHandler->listInfo()[$theme_1]->base_theme;
      // Log the themes names being compared and how we determined them.
      $this->logger->notice(dt('Comparing :theme_1 (active theme) and :theme_2 (base theme).', [
        ':theme_1' => $theme_1,
        ':theme_2' => $theme_2,
      ]));
    }

    // Get the path to the themes and log the path information.
    $theme_1_path = $this->themeHandler->getTheme($theme_1)->getPath();
    $theme_2_path = $this->themeHandler->getTheme($theme_2)->getPath();
    $this->logger->debug(dt('Themes found at :theme_1_path and :theme_2_path.', [
      ':theme_1_path' => $theme_1_path,
      ':theme_2_path' => $theme_2_path,
    ]));

    // Build the path to the theme template directory and log the information.
    $theme_1_template_path = sprintf('%s/templates', $theme_1_path);
    $theme_2_template_path = sprintf('%s/templates', $theme_2_path);
    // Search for files in each theme that match the specified template.
    $theme_1_finder = new Finder();
    $theme_1_files = $theme_1_finder->in($theme_1_template_path)->name($template)->files();
    $theme_2_finder = new Finder();
    $theme_2_files = $theme_2_finder->in($theme_2_template_path)->name($template)->files();
    // Log where the searching was done.
    $this->logger->debug(dt("Searching for template files in:\n  - :theme_1_path\n  - :theme_2_path", [
      ':theme_1_path' => $theme_1_template_path,
      ':theme_2_path' => $theme_2_template_path,
    ]));
    // Determine if a template file was found.
    $theme_1_file_path = $this->getTemplateFilePath($theme_1_files);
    $theme_2_file_path = $this->getTemplateFilePath($theme_2_files);
    // If the template was not found, log the failure and abort the command.
    if (!$theme_1_file_path) {
      $this->logger->warning(dt('Unable to find a :template template in the :theme theme.', [
        ':template' => $template,
        ':theme' => $theme_1,
      ]));
      return;
    }
    if (!$theme_2_file_path) {
      $this->logger->warning(dt('Unable to find a :template template in the :theme theme.', [
        ':template' => $template,
        ':theme' => $theme_2,
      ]));
      return;
    }
    // Log the template and themes being compared.
    $this->logger->debug(dt('Comparing the :template template in :theme2 and :theme1.', [
      ':theme1' => $theme_1,
      ':theme2' => $theme_2,
      ':template' => $template,
    ]));
    $this->logger->debug(dt("Comparing files:\n  - :path_1\n  - :path_2", [
      ':path_1' => $theme_1_file_path,
      ':path_2' => $theme_2_file_path,
    ]));

    // Build and display the diff output.
    $builder = new UnifiedDiffOutputBuilder(
      sprintf("- %s\n+ %s\n", $theme_2, $theme_1),
      FALSE,
    );
    $differ = new Differ($builder);
    print $differ->diff(
      file_get_contents($theme_2_file_path),
      file_get_contents($theme_1_file_path)
    );
  }

  /**
   * Gets the path to a template file if one was found.
   *
   * @param \Symfony\Component\Finder\Finder $files
   *   The results of a Finder operation.
   *
   * @return false|string
   *   The file path if a file was found.
   */
  protected function getTemplateFilePath(Finder $files) {
    if (empty($files)) {
      return FALSE;
    }
    foreach ($files as $file) {
      return $file->getPath() . '/' . $file->getFilename();
    }
  }

}
